using System;
using ZR.Model;
using System.Collections.Generic;
using ZR.Model.Dto.Inspect;
using ZR.Model.Models.Inspect;

namespace ZR.Service.Business.IBusinessService.Inspect
{
    /// <summary>
    /// 接口管理/SAP交验单记录表service接口
    ///
    /// @author admin
    /// @date 2023-05-05
    /// </summary>
    public interface ISapDeliverRecordService : IBaseService<SapDeliverRecord>
    {
        PagedInfo<SapDeliverRecordDto> GetList(SapDeliverRecordQueryDto parm);

        int AddSapDeliverRecord(SapDeliverRecord parm);

        int UpdateSapDeliverRecord(SapDeliverRecord parm);

        void TruncateSapDeliverRecord();

        /// <summary>
        /// 导入
        /// </summary>
        /// <param name="list"></param>
        /// <returns></returns>
        string ImportDatas(List<SapDeliverRecord> list);

        #region 2023 0524 孙亚龙 新增
        List<SapDeliverDto> GetListNoPage(SapDeliverQueryNoPageDto parm);
        List<SapDeliverRecordDto> GetSapDeliverRecordList(string deliverNo);
        SapDeliverRecordSumDto GetSapDeliverRecordSum(string materialSpecCode, string deliverNo);
        #endregion
    }
}
